/**************************************************************************
**                                                                        *
**  FILE	:  stdlib.h                                                     *
**                                                                        *
**  DESCRIPTION :  Include file with prototypes of functions for          *
**		   numeric conversions, storage allocation and exit                 *
**                                                                        *
**  Copyright 1996-2006 Altium BV                                         *
**                                                                        *
**************************************************************************/

#ifndef _STDLIB_H
#define _STDLIB_H

#ifdef	__cplusplus
extern "C" {
#endif	/* defined(__cplusplus) */

#ifndef _SIZE_T_DEF
typedef	unsigned int	size_t;
#define _SIZE_T_DEF
#endif

#ifdef	__cplusplus
#define NULL	(0)
#else	/* !defined(__cplusplus) */
#define	NULL	( 0)
#endif	/* defined(__cplusplus) */

#define EXIT_SUCCESS	0
#define EXIT_FAILURE	1
#define RAND_MAX	32767

#pragma suspend_align32
typedef struct	{
			int quot;
			int rem;
		} div_t;

typedef struct	{
			long quot;
			long rem;
		} ldiv_t;

#pragma resume_align32

#ifndef _USMLIB
#define _USMLIB
#endif

extern	_USMLIB	int                     atoi     ( const char * );
extern	_USMLIB	long                    atol     ( const char * );
extern	_USMLIB	double                  atof     ( const char * );
extern	_USMLIB	long                    strtol   ( const char *, char **, int );
extern	_USMLIB	unsigned long           strtoul  ( const char *, char **, int );
extern	_USMLIB	double                  strtod   ( const char *, char ** );
extern	_USMLIB	int                     abs      ( int );
extern	_USMLIB	long                    labs     ( long );
extern	_USMLIB	div_t                   div      ( int, int );
extern	_USMLIB	ldiv_t                  ldiv     ( long, long );
extern	_USMLIB	int                     rand     ( void );
extern	_USMLIB	void                    srand    ( unsigned int );

#if _MODEL == 'h'
extern	_USMLIB	void *                  calloc   ( unsigned long, unsigned long );
extern	_USMLIB	void *                  malloc   ( unsigned long );
extern	_USMLIB	void *                  realloc  ( void *, unsigned long );
extern	_USMLIB	void                    free     ( void * );
#else
extern	_USMLIB	void *                  calloc   ( size_t, size_t );
extern	_USMLIB	void *                  malloc   ( size_t );
extern	_USMLIB	void *                  realloc  ( void *, size_t );
extern	_USMLIB	void                    free     ( void * );
#endif
	


#if __STDC__ == 0
extern	_USMLIB	void _near *  		ncalloc ( size_t, size_t );
extern	_USMLIB	void _near *  		nmalloc ( size_t );
extern	_USMLIB	void _near *  		nrealloc( void _near *, size_t );
extern	_USMLIB	void          		nfree   ( void _near * );

#if _MODEL != 't'
extern	_USMLIB	void _far *   		fcalloc ( size_t, size_t );
extern	_USMLIB	void _far *   		fmalloc ( size_t );
extern	_USMLIB	void _far *   		frealloc( void _far *, size_t );
extern	_USMLIB	void          		ffree   ( void _far * );

extern	_USMLIB	void _shuge * 		scalloc ( size_t, size_t );
extern	_USMLIB	void _shuge * 		smalloc ( size_t );
extern	_USMLIB	void _shuge * 		srealloc( void _shuge *, size_t );
extern	_USMLIB	void          		sfree   ( void _shuge * );

extern	_USMLIB	void _huge *  		hcalloc ( unsigned long, unsigned long );
extern	_USMLIB	void _huge *  		hmalloc ( unsigned long );
extern	_USMLIB	void _huge *  		hrealloc( void _huge *, unsigned long );
extern	_USMLIB	void          		hfree   ( void _huge * );
#endif
#endif

extern	_USMLIB	void                    exit     ( int );
extern	_USMLIB	void                    abort    ( void );
extern	_USMLIB	int                     atexit   ( void (*)(void) );

extern	_USMLIB	char *                  getenv   ( const char * );
#if __STDC__ == 1
extern	_USMLIB	int                     system   ( const char * );
#endif

extern	_USMLIB	void *			bsearch  ( const void *, const void *, size_t, size_t,
					   int (*)( const void *, const void * ) );
extern	_USMLIB	void			qsort    ( void *, size_t, size_t,
					   int (*)( const void *, const void * ) );
    
#ifndef _WCHAR_T
typedef	int		wchar_t;
#define _WCHAR_T
#endif

#define MB_CUR_MAX	(1)

extern	_USMLIB	int			mblen    ( const char *, size_t );
extern	_USMLIB	int			mbtowc   ( wchar_t *, const char *, size_t );
extern	_USMLIB	int			wctomb   ( char *, wchar_t );
extern	_USMLIB	size_t			mbstowcs ( wchar_t *, const char *, size_t );
extern	_USMLIB	size_t			wcstombs ( char *, const wchar_t *, size_t );


#ifdef	__cplusplus
}
#endif	/* defined(__cplusplus) */

#endif	/* _STDLIB_H */
